/*
Example for Secure7 Click

    Date          : Oct 2019.
    Author        : Nenad Filipovic

Test configuration KINETIS :
    
    MCU              : MK64FN1M0XXX12
    Dev. Board       : Fusion for KINETIS v8
    ARM Compiler ver : v6.2.0.0

---

Description :

The application is composed of three sections :

- System Initialization - Initializes GPIO and start to write log.
- Application Initialization - Initialization driver enables - GPIO 
    and configure swi for communication, also write log.
- Application Task - (code snippet) This is an example which demonstrates the use of Secure 7 Click board.
    Data is read from the secure chip. If the read out is successful
    the data is then display on the serial port in the hex format.

Additional Functions :

- secure7_set_output - SWI directions set implementation - output.
- secure7_set_input - SWI directions set implementation - input.
- secure7_output_hex - Display output data in hex format.

*/

#include "Click_Secure7_types.h"
#include "Click_Secure7_config.h"
#include "atca_basic.h"
#include "__secure7_driver.h"

uint8_t buffer_out [ 128 ];
char log_text[ 50 ];

void secure7_set_output ( void )
{ 
    mikrobus_gpioInit( _MIKROBUS1, _MIKROBUS_RX_PIN, _GPIO_OUTPUT );
}

void secure7_set_input ( void )
{ 
    mikrobus_gpioInit( _MIKROBUS1, _MIKROBUS_RX_PIN, _GPIO_INPUT );
}

void secure7_output_hex ( uint8_t *p_output, uint8_t length )
{
    uint16_t cnt;
    
    for ( cnt = 0; cnt < length; cnt++ )
    {
        ByteToHex( p_output[ cnt ], log_text );
        ltrim( log_text );
        mikrobus_logWrite( "0x",_LOG_TEXT);
        mikrobus_logWrite( log_text, _LOG_TEXT );
        mikrobus_logWrite( " ", _LOG_TEXT );
    }
    mikrobus_logWrite( "", _LOG_LINE );
}

void system_init ( )
{
    MM_Init();
    mikrobus_gpioInit( _MIKROBUS1, _MIKROBUS_RX_PIN, _GPIO_OUTPUT );
    mikrobus_logInit( _LOG_USBUART, 9600 );
    Delay_ms( 1000 );
    
    memset( buffer_out, 0x00, 128 );
    Delay_ms( 100 );
    
    mikrobus_logWrite( "------------------------------", _LOG_LINE );
    mikrobus_logWrite( "        Secure 7 Click        ", _LOG_LINE );
    mikrobus_logWrite( "------------------------------", _LOG_LINE );
}

void application_init ( )
{
    secure7_gpio_driver_init( (secure7_obj_t)&_MIKROBUS1_GPIO );
    mikrobus_logWrite( "----  Driver Initialized  ----", _LOG_LINE );
    Delay_ms( 1000 );
    
    atcab_init( &SECURE7_ATCA_CFG );
    mikrobus_logWrite( "---- Secure 7 Initialized ----", _LOG_LINE );
    mikrobus_logWrite( "------------------------------", _LOG_LINE );
    Delay_ms( 1000 );
}

void application_task ( )
{
    if ( atcab_read_serial_number( &buffer_out[ 0 ] ) == ATCA_SUCCESS )
    {
        mikrobus_logWrite( " Serial number : ", _LOG_LINE );
        secure7_output_hex( &buffer_out[ 0 ], 9 );
    }
    else
    {
        mikrobus_logWrite( " Reading serial number failed...", _LOG_LINE );
    }
    
    mikrobus_logWrite( "----------------------------------------", _LOG_LINE );
    Delay_ms ( 1000 );

    if ( atcab_info( &buffer_out[ 0 ] )  == ATCA_SUCCESS )
    {
        mikrobus_logWrite( " Device revision information : ", _LOG_LINE );
        secure7_output_hex( &buffer_out[ 0 ], 4 );
    }
    else
    {
        mikrobus_logWrite( " Reading device revision information failed...", _LOG_LINE );
    }
    
    mikrobus_logWrite( "----------------------------------------", _LOG_LINE );
    Delay_ms ( 1000 );
    
    if ( atcab_read_config_zone( ATECC608A, &buffer_out[ 0 ] ) == ATCA_SUCCESS)
     {
        mikrobus_logWrite(" First 32 bytes of device configuration :",_LOG_LINE);
        secure7_output_hex( &buffer_out[ 0 ], 32 );
     }
     else
     {
        mikrobus_logWrite(" Reading config zone failed...",_LOG_LINE);
     }
    
    mikrobus_logWrite( "----------------------------------------", _LOG_LINE );
    Delay_ms( 10000 );
}

void main ( )
{
    system_init( );
    application_init( );

    for ( ; ; )
    {
            application_task( );
    }
}