/*
    __secure7_driver.c

-----------------------------------------------------------------------------

  This file is part of mikroSDK.

  Copyright (c) 2019, MikroElektonika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

#include "__secure7_driver.h"
#include "__secure7_hal.c"

// ------------------------------------------------------------------- MACROS 

const uint16_t SECURE7_START_PULSE_TIME_OUT = 600;
const uint16_t SECURE7_ZERO_PULSE_TIME_OUT  = 40;

const uint8_t SECURE7_ATCA_SUCCESS          = 0x00;
const uint8_t SECURE7_ATCA_RX_TIMEOUT       = 0xEB;
const uint8_t SECURE7_ATCA_INVALID_SIZE     = 0xE4;
const uint8_t SECURE7_ATCA_SMALL_BUFFER     = 0xED;
const uint8_t SECURE7_ATCA_RSP_SIZE_MIN     = 0x00;
const uint8_t SECURE7_ATCA_RX_FAIL          = 0xE6;

// ---------------------------------------------------------------- VARIABLES

#ifdef   __SECURE7_DRV_I2C__
static uint8_t dev_slave_addr;
#endif

// -------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

// SWI bitbang delays implementation
void secure7_delay_txrx ( void );
void secure7_delay_wake ( void );
void secure7_delay_1L ( void );
void secure7_delay_1H ( void );
void secure7_delay_5 ( void );
void secure7_delay_7 ( void );
void secure7_disable_irq ( void );
void secure7_enable_irq ( void );

// --------------------------------------------------------- PUBLIC FUNCTIONS 

#ifdef   __SECURE7_DRV_SPI__

void secure7_spi_driver_init(secure7_obj_t gpio_obj, secure7_obj_t spi_obj)
{
    hal_spi_map( (hal_p_t)spi_obj );
    hal_gpio_map( (hal_p_t)gpio_obj );

    // ... power ON
    // ... configure CHIP
}

#endif
#ifdef   __SECURE7_DRV_I2C__

void secure7_i2c_driver_init(secure7_obj_t gpio_obj, secure7_obj_t i2c_obj, uint8_t slave)
{
    dev_slave_addr = slave;
    hal_i2c_map( (hal_p_t)i2c_obj );
    hal_gpio_map( (hal_p_t)gpio_obj );

    // ... power ON
    // ... configure CHIP
}

#endif
#ifdef   __SECURE7_DRV_UART__

void secure7_uart_driver_init(secure7_obj_t gpio_obj, secure7_obj_t uart_obj)
{
    hal_uart_map( (hal_p_t)uart_obj );
    hal_gpio_map( (hal_p_t)gpio_obj );

    // ... power ON
    // ... configure CHIP
}

#endif

// GPIO Only Drivers - remove in other cases
void secure7_gpio_driver_init(secure7_obj_t gpio_obj)
{
    hal_gpio_map( (hal_p_t)gpio_obj );

    // ... power ON
    secure7_set_output( );
    hal_gpio_rx_set( 1 );
}

// ----------------------------------------------------------- IMPLEMENTATION 

void secure7_write_byte ( uint8_t input )
{
    hal_uart_write( input );
}

uint8_t secure7_read_byte( void  )
{
    return hal_uart_read( );
}

uint8_t secure7_byte_ready ( void )
{
    return hal_uart_ready( );
}

void secure7_send_wake_token ( void )
{
    hal_gpio_rx_set( 0 );
    secure7_delay_wake( );
    hal_gpio_rx_set( 1 );
}

void secure7_send_bytes ( uint8_t count, uint8_t *p_buf )
{
    uint8_t n_cnt;
    uint8_t bit_mask;

    secure7_set_output( );
    secure7_disable_irq( );

    for ( n_cnt = 0; n_cnt < count; n_cnt++ )
    {
        for ( bit_mask = 1; bit_mask > 0; bit_mask <<= 1 )
        {
            if ( bit_mask & p_buf[ n_cnt ] )
            {
                hal_gpio_rx_set( 0 );
                secure7_delay_1L( );
                hal_gpio_rx_set( 1 );
                secure7_delay_7( );
            }
            else
            {
                hal_gpio_rx_set( 0 );
                secure7_delay_1L( );
                hal_gpio_rx_set( 1 );
                secure7_delay_1H( );
                hal_gpio_rx_set( 0 );
                secure7_delay_1L( );
                hal_gpio_rx_set( 1 );
                secure7_delay_5( );
            }
        }
    }

    secure7_enable_irq( );
}

void secure7_send_byte ( uint8_t s_byte )
{
    secure7_send_bytes( 1, &s_byte );
}

uint8_t secure7_receive_bytes ( uint8_t count, uint8_t *p_buf )
{
    uint8_t status = SECURE7_ATCA_SUCCESS;
    uint8_t n_cnt;
    uint8_t bit_mask;
    uint8_t pulse_count;
    uint16_t timeout_count;


    secure7_set_input( );
    secure7_disable_irq( );

    // Receive bits and store in buffer.
    for ( n_cnt = 0; n_cnt < count; n_cnt++ )
    {
        p_buf[ n_cnt ] = 0;

        for ( bit_mask = 1; bit_mask > 0; bit_mask <<= 1 )
        {
            pulse_count = 0;
            timeout_count = SECURE7_START_PULSE_TIME_OUT;
            
            // Detect start bit.
            while ( --timeout_count > 0 )
            {
                // Wait for falling edge.
                asm nop;

                if ( hal_gpio_rx_get() == 0 )
                {
                    break;
                }
            }
            if ( timeout_count == 0 )
            {
                status = SECURE7_ATCA_RX_TIMEOUT;
                break;
            }

            timeout_count = SECURE7_START_PULSE_TIME_OUT;

            do
            {
                // Wait for rising edge.
                if ( hal_gpio_rx_get() != 0 )
                {
                    pulse_count = 1;
                    break;
                }
            }
            while ( --timeout_count > 0 );

            if ( pulse_count == 0 )
            {
                status = SECURE7_ATCA_RX_TIMEOUT;
                break;
            }

            //  let's just wait the maximum time for the falling edge of a zero bit
            // to arrive after we have detected the rising edge of the start bit.
            timeout_count = SECURE7_ZERO_PULSE_TIME_OUT;

            // Detect possible edge indicating zero bit.
            do
            {
                if ( hal_gpio_rx_get() == 0 )
                {
                    pulse_count = 2;
                    break;
                }
            }
            while ( --timeout_count > 0 );

            // Wait for rising edge of zero pulse before returning. Otherwise we might interpret
            // its rising edge as the next start pulse.
            if ( pulse_count == 2 )
            {
                timeout_count = SECURE7_ZERO_PULSE_TIME_OUT;

                do
                {
                    if ( hal_gpio_rx_get() != 0 )
                    {
                        break;
                    }
                }

                while ( timeout_count-- > 0 );

            }
            // Update byte at current buf index.
            else
            {
                // received "one" bit
                p_buf[ n_cnt ] |= bit_mask;
            }
        }

        if ( status != SECURE7_ATCA_SUCCESS  )
        {
            break;
        }

        if ( n_cnt == 0 )
        {
            if ( p_buf[ SECURE7_ATCA_RSP_SIZE_MIN ] < 4 )
            {
                status = SECURE7_ATCA_INVALID_SIZE;
                break;
            }
            else if ( p_buf[ 0 ] > count )
            {
                status = SECURE7_ATCA_SMALL_BUFFER;
                break;
            }
            else
            {
                count = p_buf[ 0 ] ;
            }
        }
    }

    secure7_enable_irq( );
    secure7_delay_txrx( );

    if ( status == SECURE7_ATCA_RX_TIMEOUT  )
    {
        if ( n_cnt > 0 )
        {
            // Indicate that we timed out after having received at least one byte.
            status = SECURE7_ATCA_RX_FAIL;
        }
    }

    return status;
}

// --------------------------------------------- PRIVATE FUNCTION DEFINITIONS 

/**
 * Turn around time when switching from receive to transmit should be 93 us
 *
 * @note Setting little less value as there would be other process before these steps
 */
void secure7_delay_txrx ( void )
{ 
    Delay_50us();
    Delay_22us();
    Delay_22us();
    Delay_1us();
    Delay_1us();
}

/**
 * Wake time
 */
void secure7_delay_wake ( void )
{ 
    Delay_80us();
}

/**
 * Time to keep pin low for one bit should be 4.34 us
 */
void secure7_delay_1L ( void )
{ 
    Delay_1us();
    Delay_1us();
    Delay_1us();
    Delay_1us();
}

/**
 * Time to keep pin high for one bit should be 4.34 us
 */
void secure7_delay_1H ( void )
{ 
    Delay_1us();
    Delay_1us();
    Delay_1us();
    Delay_1us();
}

/**
 * Time to keep pin high for five pulses plus stop bit should be 26.04 us
 */
void secure7_delay_5 ( void )
{ 
    Delay_22us();
    Delay_1us();
    Delay_1us();
    Delay_1us();
    Delay_1us();
}

/**
 * Time to keep pin high for seven bits plus stop bit should be 34.72 us
 */
void secure7_delay_7 ( void )
{ 
    Delay_22us();
    Delay_10us();
    Delay_1us();
    Delay_1us();
    Delay_1us();
}

void secure7_disable_irq ( void )
{ 
    /* No interrupts used in application */
}

void secure7_enable_irq ( void )
{ 
    /* No interrupts used in application */ 
}

// --------------------------------------------------------------------------

/*
  __secure7_driver.c

  Copyright (c) 2019, MikroElektonika - http://www.mikroe.com

  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
   This product includes software developed by the MikroElektonika.

4. Neither the name of the MikroElektonika nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MIKROELEKTRONIKA ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MIKROELEKTRONIKA BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

----------------------------------------------------------------------------- */