/*
    __secure7_driver.h

-----------------------------------------------------------------------------

  This file is part of mikroSDK.
  
  Copyright (c) 2019, MikroElektonika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

/**

| Global Library Prefix | **SECURE7** |
|:---------------------:|:-----------------:|
| Version               | **1.0.0**    |
| Date                  | **Oct 2019.**      |
| Developer             | **Nenad Filipovic**     |

*/

// --------------------------------------------------------------------------    

#include "stdint.h"

#ifndef _SECURE7_H_
#define _SECURE7_H_

/** 
 * @macro secure7_obj_t
 * @brief Driver Abstract type 
 */
#define secure7_obj_t    const uint8_t*

// --------------------------------------------------------------------------    

//  #define   __SECURE7_DRV_SPI__                            
//  #define   __SECURE7_DRV_I2C__                                                                      
  #define   __SECURE7_DRV_UART__                           

                                                                  
// ----------------------------------------------------------------- Variable   

/**
 * This value is decremented while waiting for the falling edge of a start pulse.
 */
extern const uint16_t SECURE7_START_PULSE_TIME_OUT;

/**
 * Maximum time between rising edge of start pulse and falling edge of zero pulse
 * is 8.6 us. Therefore, a value of 40 (around 15 us)
 *
 * Gives sample time to detect a zero pulse and also leaves enough time to detect
 * the following start pulse.
 *
 * This value is decremented while waiting for the falling edge of a zero pulse.
 */
extern const uint16_t SECURE7_ZERO_PULSE_TIME_OUT;

// -------------------------------------------------------------------- Types    



// -------------------------------------------------------------------------- 
   
#ifdef __cplusplus
extern "C"{
#endif

// ----------------------------------------------------- Driver Initialization

extern void secure7_set_output(  void );
extern void secure7_set_input(  void );

/**
 * Disable interrupts
 *
 * @note
 * This is necessary only in case of interrupts usage inside application
 */
extern void secure7_disable_irq();

/**
 * Enable interrupts
 *
 * @note
 * This is necessary only in case of interrupts usage inside application
 */
extern void secure7_enable_irq();

#ifdef   __SECURE7_DRV_SPI__
void secure7_spi_driver_init(secure7_obj_t gpio_obj, secure7_obj_t spi_obj);
#endif
#ifdef   __SECURE7_DRV_I2C__
void secure7_i2c_driver_init(secure7_obj_t gpio_obj, secure7_obj_t i2c_obj, uint8_t slave);
#endif
#ifdef   __SECURE7_DRV_UART__
void secure7_uart_driver_init(secure7_obj_t gpio_obj, secure7_obj_t uart_obj);
#endif

// GPIO Only Drivers - remove in other cases
void secure7_gpio_driver_init(secure7_obj_t gpio_obj);


// -------------------------------------------------------- Driver Functions 

/**
 * @brief Write Single Byte
 *
 * @param input  Byte for sending
 *
 * @description Writes sinle byte.
 */
void secure7_write_byte ( uint8_t input );

/**
 * @brief Read Single Byte
 *
 * @return received byte
 *
 * @description Read received byte.
 */
uint8_t secure7_read_byte( );

/**
 * @brief Check for new byte received
 *
 * @retval 0 - no new data / 1 - data ready
 *
 * @description Checks is there a new byte received.
 */
uint8_t secure7_byte_ready ( );

/**
 * @brief Send wake token function
 *
 * @description Function send wake token to
 * ATECC608A Microchip CryptoAuthentication Device
 * on the Secure 7 click.
 */
void secure7_send_wake_token();

/**
 * @brief Send bytes function
 *
 * @param count    Number of the bytes for sending
 *
 * @param p_buf    Pointer to the memory location where send data be stored
 *
 * @description Function send count number bytes of the data to
 * ATECC608A Microchip CryptoAuthentication Device
 * on the Secure 7 click.
 */
void secure7_send_bytes ( uint8_t count, uint8_t *p_buf );

/**
 * @brief Send byte function
 *
 * @param s_byte    8-bit data to send
 *
 * @description Function send the byte of the data to
 * ATECC608A Microchip CryptoAuthentication Device
 * on the Secure 7 click.
 */
void secure7_send_byte ( uint8_t s_byte );

/**
 * @brief Receive bytes function
 *
 * @param count    Number of the bytes for receiving
 *
 * @param p_buf    Pointer to the memory location where receive data be stored
 *
 * @description Function receive count number bytes of the data to
 * ATECC608A Microchip CryptoAuthentication Device
 * on the Secure 7 click.
 */
uint8_t secure7_receive_bytes ( uint8_t count, uint8_t *p_buf );

// --------------------------------------------------------------------------    

#ifdef __cplusplus
} // extern "C"
#endif
#endif

/**
    @example Click_Secure7_STM.c
    @example Click_Secure7_TIVA.c
    @example Click_Secure7_CEC.c
    @example Click_Secure7_KINETIS.c
    @example Click_Secure7_MSP.c
    @example Click_Secure7_PIC.c
    @example Click_Secure7_PIC32.c
    @example Click_Secure7_DSPIC.c
    @example Click_Secure7_AVR.c
    @example Click_Secure7_FT90x.c
    @example Click_Secure7_STM.mbas
    @example Click_Secure7_TIVA.mbas
    @example Click_Secure7_CEC.mbas
    @example Click_Secure7_KINETIS.mbas
    @example Click_Secure7_MSP.mbas
    @example Click_Secure7_PIC.mbas
    @example Click_Secure7_PIC32.mbas
    @example Click_Secure7_DSPIC.mbas
    @example Click_Secure7_AVR.mbas
    @example Click_Secure7_FT90x.mbas
    @example Click_Secure7_STM.mpas
    @example Click_Secure7_TIVA.mpas
    @example Click_Secure7_CEC.mpas
    @example Click_Secure7_KINETIS.mpas
    @example Click_Secure7_MSP.mpas
    @example Click_Secure7_PIC.mpas
    @example Click_Secure7_PIC32.mpas
    @example Click_Secure7_DSPIC.mpas
    @example Click_Secure7_AVR.mpas
    @example Click_Secure7_FT90x.mpas
**/

// --------------------------------------------------------------------------    
/*
  __secure7_driver.h

  Copyright (c) 2019, MikroElektonika - http://www.mikroe.com

  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
   This product includes software developed by the MikroElektonika.

4. Neither the name of the MikroElektonika nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MIKROELEKTRONIKA ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MIKROELEKTRONIKA BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

----------------------------------------------------------------------------- */